﻿//-----------------------------------------------------------------------------------------------------------
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//
//------------------------------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace LinqToHbaseProvider
{
    internal class ColumnFinder : ExpressionVisitor
    {
        private Expression expression;
        private List<string> columns;
        private List<string> values;

        private bool isIP;
        private bool isPage;

        public ColumnFinder(Expression exp)
        {
            this.expression = exp;
            this.isIP = false;
            this.isPage = false;
        }

        public List<string> Columns
        {
            get
            {
                if (columns == null)
                {
                    columns = new List<string>();
                    values = new List<string>();
                    this.Visit(this.expression);
                }
                return this.columns;
            }
        }

        public bool IsIP
        {
            get
            {
                return this.isIP;
            }
        }

        public bool IsPage
        {
            get
            {
                return this.isPage;
            }
        }

        protected override Expression VisitBinary(BinaryExpression be)
        {
            if (be.NodeType == ExpressionType.Equal)
            {     
                this.isIP = ExpressionTreeHelpers.IsMemberEqualsValueExpression(be, typeof(AccessRecord),  "ipAddress");
                this.isPage = ExpressionTreeHelpers.IsMemberEqualsValueExpression(be, typeof(AccessRecord), "page");
                if (this.IsIP)
                {
                    columns.Add(ExpressionTreeHelpers.GetValueFromEqualsExpression(be, typeof(AccessRecord), "ipAddress"));
                    return be;
                }
                else if (this.IsPage)
                {
                    columns.Add(ExpressionTreeHelpers.GetValueFromEqualsExpression(be, typeof(AccessRecord), "page"));
                    return be;
                }
                else
                    return base.VisitBinary(be);
            }
            else
                return base.VisitBinary(be);
        } 

        protected override Expression VisitMethodCall(MethodCallExpression m)
        {
            if (m.Method.DeclaringType == typeof(String) && m.Method.Name == "StartsWith")
            {
                this.isIP = ExpressionTreeHelpers.IsSpecificMemberExpression(m.Object, typeof(AccessRecord), "ipAddress");
                this.isPage = ExpressionTreeHelpers.IsSpecificMemberExpression(m.Object, typeof(AccessRecord), "page");
                if (this.IsIP || this.IsPage)
                {
                    columns.Add(ExpressionTreeHelpers.GetValueFromExpression(m.Arguments[0]));
                    return m;
                }

            }
            else if (m.Method.Name == "Contains")
            {
                Expression valuesExpression = null;

                if (m.Method.DeclaringType == typeof(Enumerable))
                {
                    this.isIP = ExpressionTreeHelpers.IsSpecificMemberExpression(m.Arguments[1], typeof(AccessRecord), "ipAddress");
                    this.isPage = ExpressionTreeHelpers.IsSpecificMemberExpression(m.Arguments[1], typeof(AccessRecord), "page");
                    if (this.IsIP || this.IsPage)                 
                    {
                        valuesExpression = m.Arguments[0];
                    }
                }
                else if (m.Method.DeclaringType == typeof(List<string>))
                {
                    this.isIP = ExpressionTreeHelpers.IsSpecificMemberExpression(m.Arguments[0], typeof(AccessRecord), "ipAddress");
                    this.isPage = ExpressionTreeHelpers.IsSpecificMemberExpression(m.Arguments[0], typeof(AccessRecord), "page");
                    if (this.IsIP || this.IsPage)                    
                    {
                        valuesExpression = m.Object;
                    }
                }

                if (valuesExpression == null || valuesExpression.NodeType != ExpressionType.Constant)
                    throw new InvalidQueryException("Could not find the location values.");

                ConstantExpression ce = (ConstantExpression)valuesExpression;

                IEnumerable<string> columnStrings = (IEnumerable<string>)ce.Value;
                // Add each string in the collection to the list of locations to obtain data about.
                foreach (string place in columnStrings)
                    columns.Add(place);

                return m;
            }

            return base.VisitMethodCall(m);
        }
    }
}